### The following line may have to be changed to match the local
### installation of the asypow package

library(asypow.new)

options(echo=T)
###
###----------------------------------------------------------------------
###
cat('          **********clinical.trial.covariate\n\n')

### Three Year Clinical Trial

### What is the power of a test to detect a quadratic
### term in a 3 year clinical trial with an exponential
### survival design with a significance level of
### 0.05 and a sample size of 10, 20 or 30
### patients using the likelihood ratio method?
### using the SMO method?

### First define the value of a, b and c
### under the alternative hypothesis.
abc <- c(1,1.2,.3)

### Define the length to be 3 year.
L <- 3

### Define the covariate values to be 10
### equally spaced points between -3 and 3.
covar <- seq(-3,3,length=10)

### Define a constraint which sets c to 0
const.constraint.param <- "c"
const.constraint.value <- 0
### Set the significance level and sample sizes required.
significance <- 0.05
sample.size <- c(10,20,30)

### First use the likelihood ratio method to do the
### power calculations.
method <- "lr"
ans.lr <- expsurv.design(method,abc,L,
                         design="quadratic",xpoints=covar,
                         const.constraint.param=const.constraint.param,
                         const.constraint.value=const.constraint.value,
                         significance=significance,sample.size=sample.size)
print(ans.lr)

### Next use the SMO method.
method <- "smo"
ans.smo <- expsurv.design(method,abc,L,
                          design="quadratic",xpoints=covar,
                          const.constraint.param=const.constraint.param,
                          const.constraint.value=const.constraint.value,
                          significance=significance,sample.size=sample.size)
print(ans.smo)

###
###----------------------------------------------------------------------
###
cat('          **********clinical.trial.s\n\n')

### Three Year Clinical Trial

### What is the sample size required to detect an
### actual hazard rate of 2 when the null hypothesis
### hazard rate is 1 with a power of 0.8 and a
### significance of level 0.05 assuming an
### exponential survival model using the likelihood
### ratio method? using the SMO method?

### First define rate.ha.
rate.ha <- 2

### Define the length to be 3 year.
L <- 3

### Define a constraint which sets rate to 1
const.constraint.param <- "rate"
const.constraint.value <- 1

### Set the significance level and power required.
significance <- 0.05
power <- .8

### First use the likelihood ratio method to do the
### sample size calculations.
method <- "lr"
ans.lr <- expsurv.kgp(method,rate.ha,L,
                      const.constraint.param=const.constraint.param,
                      const.constraint.value=const.constraint.value,
                      significance=significance,power=power)
print(ans.lr)

### Next use the SMO method.
method <- "smo"
ans.smo <- expsurv.kgp(method,rate.ha,L,
                       const.constraint.param=const.constraint.param,
                       const.constraint.value=const.constraint.value,
                       significance=significance,power=power)
print(ans.smo)

###
###----------------------------------------------------------------------
###
cat('          **********generic.s') 

### One-sample Binomial with Two Groups.
### Calculated using generic routine

### The two parameters of the model are the probabilities
### of an event, p[i], for each group i=1,2.

### Density Function:
### The binomial density is
### f(p) = p          if the event occurred.
###        1-p        if the event did not occur.
###
f <- function(p,ievent) {

   # p -- probability of an event.
   # ievent -- 1 if event occurred;
   #           0 if event did not occur.

   if (ievent == 1) return(p)
   else return(1-p)

}

### The Log-Likelihood Function:
### The log-likelihood is:
### l(p) = log(p)     if the event occurred.
###        log(1-p)   if the event did not occur.

l <- function(p,ievent) {

  if (ievent == 1) return(log(p))
    else return(log(1-p)) 

}

### The Expected Log-Likelihood Function:
### The expected log-likelihood of p.ho over p.ha is
### given by the density at p.ha times the log-likelihood
### at p.ho summed over the 2 possible values of ievent,
### averaged over the two groups.

expect.loglike <- function(p.ha,p.ho) {

  ell <- 0
  n <- length(p.ha)
  for (i in 1:n)
    ell <- ell + f(p.ha[i],ievent=0)*l(p.ho[i],ievent=0) +
      f(p.ha[i],ievent=1)*l(p.ho[i],ievent=1)
  return(ell/n)
}

### What is the sample size required to test the 
### hypothesis p[1] != p[2] if p.ha = (0.4,0.2) 
### using the SMO method with a power of 0.8
### and significance level of 0.01, 0.05 or 0.1?

### First define p.ha, the value of the parameters
### under the alternative hypothesis. Define p.lo
### and p.hi so that p.ha is in the range (0,1),
p.ha <- c(0.4,0.2)
p.lo <- c(0,0)
p.hi <- c(1,1)

### Define the constraints so that p[1] = p[2]
### under the null hypothesis.
equal.constraint.param <- c("theta1","theta2")

### Define the required significance level
### and power.
power <- 0.8
significance <- c(0.01,0.05,0.1)

### Calculate the sample size using the
### degrees of freedom in the formula.
ans <- generic.model.smo(theta.ha=p.ha,
                         theta.lo=p.lo,theta.hi=p.hi,
                         expect.loglike=expect.loglike,
                         equal.constraint.param=equal.constraint.param,
                         significance=significance,power=power,
                         smo.df=T)
print(ans)

### Calculate the sample size without the
### degrees of freedom for a less conservative
### estimate.
ans <- generic.model.smo(theta.ha=p.ha,
                         theta.lo=p.lo,theta.hi=p.hi,
                         expect.loglike=expect.loglike,
                         equal.constraint.param=equal.constraint.param,
                         significance=significance,power=power,
                         smo.df=F)
print(ans)

###
###----------------------------------------------------------------------
###
cat('          **********linear.binomial\n\n')

### Linear Binomial Design

### What is the sample size needed to detect a
### linear effect with a power of 0.8 and significance
### level of 0.01, 0.05 or 0.1 given that the
### value of b is actually 0.5 using the likelihood 
### ratio method? using the SMO method?

### First define ab, the actual values of
### a and b in the model.
ab <- c(1,0.5)

### Define the covariate values used in
### the experiment.
covar <- c(1,2,3,4,5)

### Define, rsamp.size, the relative sample
### sizes at each covariate value.
rsamp.size <- c(10,11,12,10,9)

### Set a constant constraint so that b = 0
### under the null hypothesis.
const.constraint.param <- "b"
const.constraint.value <- 0

### Set the significance levels and power required.
significance <- c(.01,.05,.1)
power <- .8

### First use the likelihood ratio method to do the
### sample size calculations.
method <- "lr"
ans.lr <- binomial.design(method,theta.ha=ab,
                          xpoints=covar,
                          natx=rsamp.size,
                          const.constraint.param=const.constraint.param,
                          const.constraint.value=const.constraint.value,
                          significance=significance,power=power)
print(ans.lr)

### Next use the SMO method.
method <- "smo"
ans.smo <- binomial.design(method,theta.ha=ab,
                           xpoints=covar,
                           natx=rsamp.size,
                           const.constraint.param=const.constraint.param,
                           const.constraint.value=const.constraint.value,
                           significance=significance,power=power)
print(ans.smo)

###
###----------------------------------------------------------------------
###
cat('          **********linear.ordinal\n\n')

### Two group Linear Ordinal Design with Three Categories

### What is the sample size needed to detect unequal
### linear effects with a power of 0.8 and a significance 
### level of 0.05 in a two group ordinal design
### with three categories using the likelihood ratio method?
### using the SMO method?

### Assume that actual parameters, (a1,a2,b), are
### (1,2,.5) for the first group and (1.5,2.5,1.0)
### for the second group.
theta.ha <- rbind(c(1,2,.5),
                  c(1.5,2.5,1.0))

### Assume that the experiment has 20 equally
### spaced design points between -10 and 10
xpoints <- seq(-10,10,length=20)

### Set the equality constraint so that b is the
### same in groups 1 and 2.
equal.constraint.param <- c("b","b")
equal.constraint.group <- c(1,2)

### Set the significance levels and power required.
significance <- 0.05
power <- .8

### First use the likelihood ratio method to do the
### sample size calculations.
method <- "lr"
ans.lr <- ordinal.design(method,theta.ha,
                         xpoints=xpoints,
                         equal.constraint.group=equal.constraint.group,
                         equal.constraint.param=equal.constraint.param,
                         significance=significance,power=power)
print(ans.lr)

### Next use the SMO method.
method <- "smo"
ans.smo <- ordinal.design(method,theta.ha,
           xpoints=xpoints,
           equal.constraint.group=equal.constraint.group,
           equal.constraint.param=equal.constraint.param,
           significance=significance,power=power)
print(ans.smo)

###
###----------------------------------------------------------------------
###
cat('          **********multinomial\n\n')
source('multinomial.s')

### Multinomial Model with Two Categories

### What is the power of a test to determine
### if the two category probabilities are equal
### in a multinomial model with a significance
### level of 0.05 and a sample size of 10, 20 or
### 30 using the likelihood ratio method?
### using the SMO method?

### Assume the first category probability is 0.4
### The second and final category probability is 
### 1-0.4=0.6 and does not need to specified.
p.ha <- 0.4

### Set a constraint so that the probabilities
### in the 2 categories are equal. This is
### the same as requiring the first category
### probability to be 0.5.
const.constraint.param <- "p1"
const.constraint.value <- 0.5

### Set the sample size and significance level.
sample.size <- c(10,20,30)
significance <- .05

### First use the likelihood ratio method to do the
### power calculations.
method <- "lr"
ans.lr <- multinomial.kgp(method,p.ha,
                          const.constraint.param=const.constraint.param,
                          const.constraint.value=const.constraint.value,
                          significance=significance,sample.size=sample.size)
print(ans.lr)

### Next use the SMO method.
### For a less conservative calculations set smo.df
### to FALSE.
method <- "smo"
smo.df <- F
ans.smo <- ordinal.kgp(method,p.ha,
                       const.constraint.value=const.constraint.value,
                       const.constraint.param=const.constraint.param,
                       significance=significance,sample.size=sample.size,
                       smo.df=smo.df)
print(ans.smo)

###
###----------------------------------------------------------------------
###
cat('          **********multivariate.logistic\n\n')

### Interaction in a Multivariate Logistic Design

### Suppose there are 2 treatments to help a patient
### stop smoking, A and B. The probability of stopping
### is a logistic function of the variables x1,x2,x3
### and x4 so that p = exp(u)/(1+exp(u))
###    u = coef1*x1 + coef2*x2 + coef3*x3 + coef4*x4
### x1 is always 1 and coef1 is an intercept term;
### x2 is 1 if the patient receives treatment A and
### coef2 is the effect of treatment A; x3 is 1 if
### the patient receives treatment B and coef3 is
### the effect of treatment B; x4 is 1 if the
### patient receives both treatments A and B and coef4
### is the interaction effect of between treatment A
### and B.

### What is the power of a study to determine if there
### is an interaction between treatment A and B given 
### 1000 patients equally split between 4 treatment groups: 
### no treatment, treatment A only, treatment B only and 
### both treatment A and B?

### Assume the coefficient values under the alternative
### hypothesis are (1,.8,.9,1.2)
coef.ha <- c(1,.8,.9,1.2)

### Set the design points defining the four
### treatment groups in the matrix xpoints.
xpoints <- rbind(c(1,0,0,0),     # no treatment
                 c(1,1,0,0),     # A only
                 c(1,0,1,0),     # B only
                 c(1,1,1,1))     # A and B
                 
### Set a constant constraint so that coef4 is
### 0 under the null hypothesis, i.e. there is
### no interaction effect between A and B.
const.constraint.param <- "coef4"
const.constraint.value <- 0

### Assume there are 1,000 patients and we require
### a significance level of 0.05.
sample.size <- 1000
significance <- 0.05

### First use the likelihood ratio method to do the
### power calculation.
method <- "lr"
ans.lr <- mvlogistic.design(method,coef.ha,xpoints,
                            const.constraint.param=const.constraint.param,
                            const.constraint.value=const.constraint.value,
                            significance=significance,sample.size=sample.size)
print(ans.lr)

### Next use the SMO method.
method <- "smo"
ans.smo <- mvlogistic.design(method,coef.ha,xpoints,
                             const.constraint.param=const.constraint.param,
                             const.constraint.value=const.constraint.value,
                             significance=significance,sample.size=sample.size)
print(ans.smo)

###
###----------------------------------------------------------------------
###
cat('          **********multivariate.loglin\n\n')

### Multivariate Log-Linear Design

### Suppose there are 2 treatments to help a patient
### stop smoking, A and B. The probability of stopping
### is a function of the variables x1,x2 and x3
### such that p = exp(u)
###    u = log(coef1)*x1 + log(coef2)*x2 + 
###        log(coef3)*x3
### x1 is always 1 and coef1 is an intercept term;
### x2 is 1 if the patient receives treatment A and
### coef2 is the effect of treatment A; x3 is 1 if
### the patient receives treatment B and coef3 is
### the effect of treatment B.

### What is the power of a study to determine if the
### if the effect of treatment A and B are equal given 
### 100 patients split between 3 treatment groups: 
### no treatment, treatment A and treatment B?

### Assume the coefficient values under the alternative
### hypothesis are (.8,.5,.9)
coef.ha <- c(.8,.5,.9)

### Set the design points defining the three
### treatment groups in the matrix xpoints.
xpoints <- rbind(c(1,0,0),     # no treatment
                 c(1,1,0),     # treatment A
                 c(1,0,1))     # treatment B
                
### Assume that there is 1 patient at design
### point 1 for every 2 patient at design
### points 2 and 3
rss <- c(1,2,2)
 
### Set an equality constraint so that coef2
### = coef3 under the null hypothesis.
equal.constraint.param <- c("coef2","coef3")

### Assume there are 100 patients and we require
### a significance level of 0.05.
sample.size <- 100
significance <- 0.05

### First use the likelihood ratio method to do the
### power calculation.
method <- "lr"
ans.lr <- mvloglin.design(method,coef.ha,xpoints,rss,
                          equal.constraint.param=equal.constraint.param,
                          significance=significance,sample.size=sample.size)
print(ans.lr)

### Next use the SMO method.
method <- "smo"
ans.smo <- mvloglin.design(method,coef.ha,xpoints,rss,
                           equal.constraint.param=equal.constraint.param,
                           significance=significance,sample.size=sample.size)
print(ans.smo)

###
###----------------------------------------------------------------------
###
cat('          **********one.sample.binomial\n\n')

### One-sample Binomial with Two Groups

### The two parameters of the model are the probability 
### of an event, p[i], in each group i=1,2.

### What is the sample size needed to detect p[1] != p[2]
### with a power of 0.8 and significance level of 0.01,
### 0.05 or 0.1 given that the actual probabilities
### are 0.3 and 0.4 using the likelihood ratio method?
### using the SMO method?

### First define p.ha
p.ha <- matrix(c(0.4,0.3),ncol=1)

### Set an equality constraint so that "p" in group 1 
### is equal to "p" in group 2.
equal.constraint.group <- c(1,2)
equal.constraint.param <- c("p","p")

### Define group.size assuming that there are 10 patients 
### in the first group for every 9 patients in the second group.
group.size <- c(10,9)

### Set the significance levels and power required.
significance <- c(.01,.05,.1)
power <- .8

### First use the likelihood ratio method to do the
### sample size calculations.
method <- "lr"
ans.lr <- binomial.kgp(method,p.ha,group.size,
                       equal.constraint.group=equal.constraint.group,
                       equal.constraint.param=equal.constraint.param,
                       significance=significance,power=power)
print(ans.lr)

### Next use the SMO method.
method <- "smo"
ans.smo <- binomial.kgp(method,p.ha,group.size,
                        equal.constraint.group=equal.constraint.group,
                        equal.constraint.param=equal.constraint.param,
                        significance=significance,power=power)
print(ans.smo)

###
###----------------------------------------------------------------------
###
cat('          **********three.group.poisson\n\n')

### Three Group Poisson Model

### What is the sample size required to detect
### unequal means in a 3 group poisson model
### with actual means (1,1.5,1.3) with a 
### significance level of 0.05 and a power of
### 0.7, 0.8 or 0.9 using the likelihood ratio
### method? using the SMO method?

### First define lambda.ha
lambda.ha <- matrix(c(1,1.5,1.3),ncol=1)

### Define a constraint which requires
### the means of all 3 groups to be equal.
equal.constraint.group <- c(1,2,3)

### Set the significance level and power required.
significance <- 0.05
power <- c(0.7,0.8,0.9)

### First use the likelihood ratio method to do the
### sample size calculations.
method <- "lr"
ans.lr <- poisson.kgp(method,lambda.ha,
                      equal.constraint.group=equal.constraint.group,
                      significance=significance,power=power)
print(ans.lr)

### Next use the SMO method.
method <- "smo"
ans.smo <- poisson.kgp(method,lambda.ha,
                       equal.constraint.group=equal.constraint.group,
                       significance=significance,power=power)
print(ans.smo)

###
###----------------------------------------------------------------------
###
cat('          **********two.group.linear.poisson\n\n')

### Two Group Linear Poisson Design
###
### What is the sample size needed to detect unequal
### linear effects in a 2 group linear Poisson design
### with a power 0.8 and significance level
### of 0.01, 0.05 or 0.1 given that the value
### of b is actually 0.1 in group 1 and 0.3 in
### group 2 using the likelihood ratio method? 
### using the SMO method?

### First define ab, the actual values of
### a and b for both groups.
ab <- rbind(c(1,0.1),c(1.5,0.3))

### Define the covariate values used in
### the experiment to be 10 equally
### spaced points between -5 and 5.
covar <- seq(-5,5,length=10)

### Define a constraint which sets "b" in group
### 1 equal to "b" in group 2.
equal.constraint.param <- c("b","b")
equal.constraint.group <- c(1,2)

### Set the significance levels and power required.
significance <- c(.01,.05,.1)
power <- .8

### First use the likelihood ratio method to do the
### sample size calculations.
method <- "lr"
ans.lr <- poisson.design(method,theta.ha=ab,
          xpoints=covar,
          equal.constraint.group=equal.constraint.group,
          equal.constraint.param=equal.constraint.param,
          significance=significance,power=power)
print(ans.lr)

### Next use the SMO method.
method <- "smo"
ans.smo <- poisson.design(method,theta.ha=ab,
                          xpoints=covar,
                          equal.constraint.group=equal.constraint.group,
                          equal.constraint.param=equal.constraint.param,
                          significance=significance,power=power)
print(ans.smo)

###
###----------------------------------------------------------------------
###
cat('          **********two.group.ordinal\n\n')

### Two Group Ordinal Model with Five Categories

### What is the significance level obtained when
### attempting to detect unequal category probabilities 
### between two groups using a five category ordinal 
### model with a sample size of 50 and power of 0.8
### using the likelihood ratio method? using the SMO
### method?

### First assume the two probability vectors are
### (0.1,0.3,0.5,0.7,1.0) and (0.2,0.4,0.6,0.8,1.0)
p.ha <- rbind(c(0.1,0.3,0.5,0.7),c(0.2,0.4,0.6,0.8))

### Set group.size assuming there are 2 subjects in group
### 1 for every subject in group 2 in the experiment.
group.size <- c(2,1)

### Set equality constraints so that "p1", "p2"
### "p3" and "p4" are equal in groups 1 and 2.
equal.constraint.group <- list(c(1,2),c(1,2),c(1,2),c(1,2))
equal.constraint.param <- list(c("p1","p1"),c("p2","p2"),
                               c("p3","p3"),c("p4","p4"))

### Set the sample size and power.
sample.size <- 50
power <- .8

### First use the likelihood ratio method to do the
### significance level calculations.
method <- "lr"
ans.lr <- ordinal.kgp(method,p.ha,group.size,
                      equal.constraint.group=equal.constraint.group,
                      equal.constraint.param=equal.constraint.param,
                      power=power,sample.size=sample.size)
print(ans.lr)

### Next use the SMO method.
### For a less conservative calculations set smo.df
### to FALSE.
method <- "smo"
smo.df <- F
ans.smo <- ordinal.kgp(method,p.ha,group.size,
                       equal.constraint.group=equal.constraint.group,
                       equal.constraint.param=equal.constraint.param,
                       power=power,sample.size=sample.size,smo.df=smo.df)
print(ans.smo)

